#
#	photozou.rb: show photographs published in Photozou
#
#	Copyright (C) 2008 by Nishimoto Masaki <gaju@gaju.jp>
#	Distributed under GPL.
#
require 'open-uri'
require 'timeout'
require 'rexml/document'
require 'fileutils'

#
#	Constants
#
Photozou_default_api_url	= 'http://api.photozou.jp/rest/photo_info'
Photozou_default_file		= "#{@cache_path}/photozou.dat"
Photozou_default_items = {
	'title'		=> [ 'photo_title', nil ],
	'url'			=> [ 'url', nil ],
	'regular'	=> [ 'image_url', nil ],
	'original'	=> [ 'original_image_url', nil ],
	'thumbnail'	=> [ 'thumbnail_image_url', nil ],
}
Photozou_xml_root					= '//rsp/info/photo'
Photozou_default_image_size	= 'regular'
Photozou_image_sizes				= [ 'thumbnail', 'regular', 'original' ]

#
#	CLASS: Photozou
#
class Photozou
	attr_reader :photo_id, :error, :data

	def parse_xml( conf, xmldoc )
		root = xmldoc.elements.to_a( Photozou_xml_root )[0]
		conf['photozou.items'].each do | kwd, path |
			tag = root.elements.to_a( "#{path[0]}" )[0]
			val = path[1] ? tag.attributes[ "#{path[1]}" ] : tag.text
			data[kwd] = conf.to_native( val ) if val
		end
	end

	def initialize
		@photo_id = nil
		@data = Hash.new
		@error = nil
	end

	def get( conf, photo_id )
		@photo_id = photo_id
		api_url = conf['photozou.api_url'].gsub(/[\t\n]/, '')
		api_url =~ %r!http://([^/]+)(.*)!
		api_url =  %Q!http://#{$1}#{$2}?photo_id=#{@photo_id}!
		@error = nil
		begin
			xml = nil
			proxy = conf['proxy']? ( 'http://' + conf['proxy'] ) : nil
			timeout( 10 ) do
				xml = open( api_url, :proxy => proxy ) {|f| f.read }
			end
			if xml then
				xmldoc = REXML::Document::new( xml )
				if xmldoc then
					parse_xml( conf, xmldoc )
				end
			end
		rescue Timeout::Error, StandardError
			@error = conf.to_native( $!.message.gsub( /[\t\n]/, ' ' ) )
		end
		self
	end

	def to_s
		r = "#{@photo_id}\t#{@error}"
		@data.each do | item, value |
			r << "\t#{item}\t#{value}" if value and not value.empty?
		end
		r << "\n"
	end

	def parse( string )
		i = string.chomp.split( /\t/ )
		@photo_id = i.shift
		error = i.shift
		@error = ( error and not error.empty? )? error : nil
		@data.clear
		while not i.empty? do
			@data[i.shift] = i.shift
		end
		self
	end

	def to_html( conf, size )
		@error ? ( conf['photozou.show_error'] ? error_html_string : '' ) : html_string( conf, size )
	end

	def to_i_html( conf, size )
		@error ? '' : i_html_string( conf, size )
	end

	def store( file )
		ddir = File.dirname( file )
		FileUtils::mkdir_p( ddir )
		begin
			data = File::open( file ) {|fh| fh.read }
		rescue Errno::ENOENT
		end
		File::open( file, 'w' ) do |fh|
			added = false
			if data then
				data.each( "\n" ) do |l|
					if /^#{@photo_id}\t/ =~ l then
						fh.puts( to_s ) if not added
						added = true
					else
						fh.puts( l )
					end
				end
			end
			fh.puts( to_s ) if not added
		end
	end

	class << self
		def parse( string )
			new.parse( string )
		end

		def restore( file, photo_id )
			r = nil
			begin
				File::open( file, 'r' ) do |fh|
					fh.each( "\n" ) do |l|
						if /^#{photo_id}\t/ =~ l then
							r = l
							break
						end
					end
				end
			rescue Errno::ENOENT
			end
			r ? Photozou::parse( r ) : nil
		end
	end
end

#
#	CLASS: Photozou (display methods)
#
class Photozou
	include ERB::Util

	def error_html_string
		%Q!<span class="error">Error: #{h( @error )}</span>!
	end

	def html_string( conf, size )
		r = %Q!<a href="#{h( @data['url'] )}">!
		item_name = ( size and not size.empty? and @data.include?(size) )? size : Photozou_default_image_size
		image_url = @data[item_name]
		r << %Q!<img src="#{h( image_url )}" alt="#{h( @data['title'] )}" title="#{h( @data['title'] )}" class="photozou">!
		r << "</a>\n"
	end

	def i_html_string( conf, size )
		r = %Q!<a href="#{h( @data['url'] )}">#{h( @data['title'] )}</a>\n!
	end
end

#
#	functions
#
def photozou( photo_id, size = nil )
	photozou_init
	size = @conf['photozou.image_size'] if not size or size.empty?
	if photo_id and not photo_id.empty? then
		w = Photozou::restore( @conf['photozou.file'], photo_id )
		if not w or w.error then
			w = Photozou.new
			w.get( @conf, photo_id )
			w.store( @conf['photozou.file'] )
		end
		if w then
			unless @cgi.mobile_agent? then
				w.to_html( @conf, size )
			else
				w.to_i_html( @conf, size )
			end
		else
			''
		end
	else
		''
	end
end

def photozou_init
	if @conf['photozou.show_error'] == nil then
		@conf['photozou.show_error'] = false
	end
	@conf['photozou.api_url']    ||= Photozou_default_api_url
	@conf['photozou.file']       ||= Photozou_default_file
	@conf['photozou.items']      ||= Photozou_default_items
	@conf['photozou.image_size'] ||= Photozou_default_image_size
end

def saveconf_photozou
	if @mode == 'saveconf' then
		@conf['photozou.image_size'] = @cgi.params['photozou.image_size'][0]
		@conf['photozou.show_error'] = @cgi.params['photozou.show_error'][0] ? true : false
		if @cgi.params['photozou.remove_cache'][0] then
			begin
				File::unlink( @conf['photozou.file'] )
			rescue Errno::ENOENT
			end
		end
	end
end

def configure_photozou
	saveconf_photozou
	photozou_init
	photozou_configure_html
end

def photozou_configure_html
	size_options = ''
	Photozou_image_sizes.each do | size |
		selected = ( size == @conf['photozou.image_size']) ? 'selected' : ''
		size_options << %Q!<option value="#{size}" #{selected}>#{size}</option>!
	end
	<<-HTML
	<h3 class="subtitle">#{@photozou_plugin_name}</h3>
	<p>#{@photozou_plugin_description}</p>
	<h3 class="subtitle">#{@photozou_image_size_label}</h3>
 	<p><select name="photozou.image_size">#{size_options}</select></p>
	<h3 class="subtitle">#{@photozou_remove_cache_label}</h3>
 	<p><input type="checkbox" name="photozou.remove_cache">#{@photozou_remove_cache_description}</p>
	HTML
end

#
#	hooks
#
add_conf_proc( 'photozou', @photozou_plugin_name ) do configure_photozou end

# photozou.rb ends here
